#include<avr/io.h>
#include"MMC_SD/MMC_SD.h"
#include"FAT/FAT.h"
#include <util/delay.h>

// control line pins
#define	LCD_CD	PC7   
#define	LCD_WR  PC6
#define	LCD_RD  PC5
#define	LCD_CS	PC4
#define	LCD_RES	PC2

// macros for control lines
#define CLR_CD PORTC &= ~(1<<LCD_CD);
#define SET_CD PORTC |=  (1<<LCD_CD);

#define CLR_CS PORTC &= ~(1<<LCD_CS);
#define SET_CS PORTC |=  (1<<LCD_CS);

#define CLR_RESET PORTC &= ~(1<<LCD_RES);
#define SET_RESET PORTC |=  (1<<LCD_RES);

#define CLR_WR PORTC &= ~(1 << LCD_WR);
#define SET_WR PORTC |=  (1 << LCD_WR);
#define CLR_RD PORTC &= ~(1 << LCD_RD);
#define SET_RD PORTC |=  (1 << LCD_RD);

// color definitions
#define	BLACK	  0x0000
#define	BLUE	  0x001F
#define	RED 	  0xF800
#define	GREEN   0x07E0
#define CYAN	  0x07FF
#define MAGENTA 0xF81F
#define YELLOW  0xFFE0
#define WHITE	  0xFFFF

#define uint8 unsigned char
#define uint16 unsigned int
#define uint32 unsigned long

unsigned char * PATH = "\\batch";

extern uint16 SectorsPerClust;
extern uint16 FirstDataSector; 
extern uint8 FAT32_Enable;

struct FileInfoStruct FileInfo;

struct direntry PictureInfo;
uint16 totalfiles = 0;

/*************************************************/
void delay(unsigned int t)
{
	unsigned char t1;
	while(t--)
	for(t1=11;t1>0;t1--)
	{asm("nop");}
}
/*************************************************/
void write_command(unsigned int command)
{
	CLR_CS;
	CLR_CD;
	SET_RD;
	SET_WR;

	PORTA=command;
	PORTE=command>>8;

	CLR_WR;
	SET_WR;
	SET_CS;
}

/*************************************************/
void write_data(unsigned int data)
{
	CLR_CS;
	SET_CD;
	SET_RD;
	SET_WR;

	PORTA=data;
	PORTE=data>>8;

	CLR_WR;
	SET_WR;
	SET_CS;

	// keeps the display from going wonky -- don't know why this is.
	PORTA=0xFFFF;
	PORTE=0xFFFF;

}
/*************************************************/
void initialization()
{
	SET_RD;
	SET_WR;
	SET_CS;
	SET_CD;
	PORTA=0x00;
	PORTE=0x00;

	CLR_RESET;
	delay(10000);
	delay(10000);
	SET_RESET;
	delay(500);

	write_command(0x0606);    // Pin Control (R606h)
	write_data(0x0000);       // Page 41 of SPFD5420A Datasheet
	delay(50);
	write_command(0x0007);    // Display Control 1 (R007h)
	write_data(0x0001);       // Page 16 of SPFD5420A Datasheet 
	delay(50);
	write_command(0x0110);    // Power Control 6(R110h)
	write_data(0x0001);       // Page 30 of SPFD5420A Datasheet
	delay(50);
	write_command(0x0100);    // Power Control 1 (R100h)
	write_data(0x17B0);       // Page 26 of SPFD5420A Datasheet
	write_command(0x0101);    // Power Control 2 (R101h)
	write_data(0x0147);       // Page 27 of SPFD5420A Datasheet
	write_command(0x0102);    // Power Control 3 (R102h)
	write_data(0x019D);       // Page 28 of SPFD5420A Datasheet
	write_command(0x0103);    // Power Control 4 (R103h)
	write_data(0x3600);       // Page 29 of SPFD5420A Datasheet
	write_command(0x0281);    // NVM read data 2 (R281h)  
	write_data(0x0010);       // Page 34 of SPFD5420A Datasheet
	delay(50);
	write_command(0x0102);    // Power Control 3 (R102h)
	write_data(0x01BD);       // Page 28 of SPFD5420A Datasheet
	delay(50);

	//--------------- Power control 1~6 ---------------//
	write_command(0x0100);    // Power Control 1 (R100h)
	write_data(0x16B0);       // Page 26 of SPFD5420A Datasheet
	write_command(0x0101);    // Power Control 2 (R101h)
	write_data(0x0147);       // Page 27 of SPFD5420A Datasheet
	write_command(0x0102);    // Power Control 3 (R102h)
	write_data(0x01BD);       // Page 28 of SPFD5420A Datasheet
	write_command(0x0103);    // Power Control 4 (R103h)
	write_data(0x2d00);       // Page 29 of SPFD5420A Datasheet
	write_command(0x0107);    // Power Control 5 (R107h)
	write_data(0x0000);       // Page 30 of SPFD5420A Datasheet
	write_command(0x0110);    // Power Control 6(R110h)
	write_data(0x0001);       // Page 30 of SPFD5420A Datasheet
	write_command(0x0280);    // NVM read data 1 (R280h)
	write_data(0x0000);       // Page 33 of SPFD5420A Datasheet
	write_command(0x0281);    // NVM read data 2 (R281h)
	write_data(0x0006);       // Page 34 of SPFD5420A Datasheet
	write_command(0x0282);    // NVM read data 3 (R282h)
	write_data(0x0000);       // Page 34 of SPFD5420A Datasheet

	//------- Gamma 2.2 control (R300h to R30Fh) ------//
	write_command(0x0300); 
	write_data(0x0101);    
	write_command(0x0301); 
	write_data(0x0b27);    
	write_command(0x0302); 
	write_data(0x132a);    
	write_command(0x0303); 
	write_data(0x2a13);    
	write_command(0x0304); 
	write_data(0x270b);    
	write_command(0x0305); 
	write_data(0x0101);    
	write_command(0x0306); 
	write_data(0x1205);    
	write_command(0x0307); 
	write_data(0x0512);    
	write_command(0x0308); 
	write_data(0x0005);    
	write_command(0x0309); 
	write_data(0x0003);    
	write_command(0x030A); 
	write_data(0x0f04);    
	write_command(0x030B); 
	write_data(0x0f00);    
	write_command(0x030C); 
	write_data(0x000f);    
	write_command(0x030D);    
	write_data(0x040f);       
	write_command(0x030E);    
	write_data(0x0300);       
	write_command(0x030F);    
	write_data(0x0500);       

	write_command(0x0400);    // Base Image Number of Line (R400h)
	write_data(0x3500);       // Page 36 of SPFD5420A Datasheet
	write_command(0x0401);    // Base Image Display Control (R401h)
	write_data(0x0001);       // Page 39 of SPFD5420A Datasheet
	write_command(0x0404);    // Based Image Vertical Scroll Control (R404h)
	write_data(0x0000);       // Page 40 of SPFD5420A Datasheet

	//--------------- Normal set ---------------//
	write_command(0x0000);    // ID Read Register (R000h)
	write_data(0x0000);       // Page 13 of SPFD5420A Datasheet
	write_command(0x0001);    // Driver Output Control Register (R001h)
	write_data(0x0100);       // Page 14 of SPFD5420A Datasheet
	write_command(0x0002);    // LCD Driving Waveform Control (R002h)
	write_data(0x0100);       // Page 14 of SPFD5420A Datasheet
	write_command(0x0003);    // Entry Mode (R003h)
	write_data(0x1030);       // Page 15 of SPFD5420A Datasheet
	write_command(0x0006);    // Display Control 1 (R007h)
	write_data(0x0000);       // Page 16 of SPFD5420A Datasheet	
	write_command(0x0008);    // Display Control 2 (R008h)
	write_data(0x0808);       // Page 17 of SPFD5420A Datasheet
	write_command(0x0009);    // Display Control 3 (R009h)
	write_data(0x0001);       // Page 18 of SPFD5420A Datasheet
	write_command(0x000B);    // Low Power Control (R00Bh)
	write_data(0x0010);       // Page 19 of SPFD5420A Datasheet
	write_command(0x000C);    // External Display Interface Control 1 (R00Ch)
	write_data(0x0000);       // Page 19 of SPFD5420A Datasheet
	write_command(0x000F);    // External Display Interface Control 2 (R00Fh)
	write_data(0x0000);       // Page 20 of SPFD5420A Datasheet
	write_command(0x0007);    // Display Control 1 (R007h)
	write_data(0x0001);       // Page 16 of SPFD5420A Datasheet

	//--------------- Panel interface control 1~6 ---------------//
	write_command(0x0010);    // Panel Interface Control 1 (R010h)
	write_data(0x0012);       // Page 20 of SPFD5420A Datasheet
	write_command(0x0011);    // Panel Interface Control 2 (R011h)
	write_data(0x0202);       // Page 21 of SPFD5420A Datasheet
	write_command(0x0012);    // Panel Interface control 3 (R012h)
	write_data(0x0300);       // Page 22 of SPFD5420A Datasheet
	write_command(0x0020);    // Panel Interface control 4 (R020h)
	write_data(0x021E);       // Page 22 of SPFD5420A Datasheet
	write_command(0x0021);    // Panel Interface Control 5 (021Rh)
	write_data(0x0202);       // Page 24 of SPFD5420A Datasheet
	write_command(0x0022);    // Panel Interface Control 6 (R022h)
	write_data(0x0100);       // Page 25 of SPFD5420A Datasheet
	write_command(0x0090);    // Frame Marker Control (R090h)
	write_data(0x8000);       // Page 25 of SPFD5420A Datasheet

	//--------------- Partial display ---------------//
	write_command(0x0210);    // Window Horizontal RAM Address Start (R210h)
	write_data(0x0000);       // Page 35 of SPFD5420A Datasheet
	write_command(0x0211);    // Window Horziontal RAM Address End (R211h)
	write_data(0x00EF);       // Page 35 of SPFD5420A Datasheet
	write_command(0x0212);    // Window Vertical RAM Address Start (R212h)
	write_data(0x0000);       // Page 35 of SPFD5420A Datasheet
	write_command(0x0213);    // Window Vertical RAM Address End (R213h)
	write_data(0x018F);       // Page 35 of SPFD5420A Datasheet
	write_command(0x0500);    // Display Position - Partial Display 1 (R500h)
	write_data(0x0000);       // Page 40 of SPFD5420A Datasheet
	write_command(0x0501);    // RAM Address Start - Partial Display 1 (R501h)
	write_data(0x0000);       // Page 40 of SPFD5420A Datasheet
	write_command(0x0502);    // RAM Address End - Partail Display 1 (R502h)
	write_data(0x0000);       // Page 40 of SPFD5420A Datasheet
	write_command(0x0503);    // Display Position - Partial Display 2 (R503h)
	write_data(0x0000);       // Page 40 of SPFD5420A Datasheet
	write_command(0x0504);    // RAM Address Start  Partial Display 2 (R504h)
	write_data(0x0000);       // Page 41 of SPFD5420A Datasheet
	write_command(0x0505);    // RAM Address End  Partial Display 2 (R505h)
	write_data(0x0000);       // Page 41 of SPFD5420A Datasheet
	write_command(0x0606);    // Pin Control (R606h)
	write_data(0x0000);       // Page 41 of SPFD5420A Datasheet
	write_command(0x06F0);    // NVM Access Control (R6F0h)
	write_data(0x0000);       // Page 41 of SPFD5420A Datasheet

	write_command(0x0007);    // Display Control 1 (R007h)
	write_data(0x0173);       // Page 16 of SPFD5420A Datasheet
	delay(50);
	write_command(0x0007);    // Display Control 1 (R007h)
	write_data(0x0171);       // Page 16 of SPFD5420A Datasheet
	delay(10);
	write_command(0x0007);    // Display Control 1 (R007h)
	write_data(0x0173);       // Page 16 of SPFD5420A Datasheet

}

void Display_Home()
{
	write_command(0x0200);    // GRAM Address Set (Horizontal Address) (R200h)
	write_data(0x0000);       // Page 30 of SPFD5420A Datasheet
	write_command(0x0201);    // GRAM Address Set (Vertical Address) (R201h)
	write_data(0x0000);       // Page 30 of SPFD5420A Datasheet

	write_command(0x0202);    // Write Data to GRAM (R202h)
}


void display_rgb(unsigned int data)
{
	unsigned int i,j;

	for(i=0;i<400;i++)
	{
		for(j=0;j<240;j++)
		{
			write_data(data);
		}
	}
}

void Picture_Slide_Show()
{
	uint8 i;              //loop variable
	uint16 j;             //loop variable
	uint32 p;             //cluster
	uint16 *buffer;				//buffer
	uint16 pics = 1;
	uint16 slide_show_flag = 1;
	uint8 sector;
	uint32 pixels;

	if (totalfiles == 0)
	return;

	buffer = malloc(512);
	if (buffer == 0)
	return;

next: 	//label for "goto"
	display_rgb(0x0000);

	Search(PATH, &PictureInfo, &pics);                                         //find the file
	p = PictureInfo.deStartCluster + (((uint32)PictureInfo.deHighClust) << 16);//the first cluster of the file

	sector = 0;

	FAT_LoadPartCluster(p, sector, buffer);	//read a sector


	pixels = (uint32) 240 * 400; // total # of pixels to write

	Display_Home();

	j = 0; // byte count
	
	while (1)
	{
		write_data(buffer[j]);  // write 16 bits
		pixels--;               // which is one pixel
		j++;                    // word count

		if (pixels == 0)        
		break;

		if (j == 256)           // time for a new sector     
		{
			sector++;
			if (sector == SectorsPerClust)
			{
				p = FAT_NextCluster(p);				// read next cluster
				sector = 0;
			}

			FAT_LoadPartCluster(p, sector, buffer);	// read a sector
			j = 0;
		}
	}

	if (slide_show_flag)
	{
		for (i=0;i<30;i++)	// delay for a while
		delay(0xffff);

		pics++;					// increment picture number
		if (pics > totalfiles)	// if last
		pics = 1;			// wrap around

		goto next;				// go show next pic
	}
	free(buffer);
}


void LCD_test()
{

	unsigned int i,j;
	Display_Home();

	for(i=0;i<240;i++)
	for(j=0;j<400;j++)
	{
		if(i>209)write_data(BLACK);
		else if(i>179)write_data(BLUE);
		else if(i>149)write_data(RED);
		else if(i>119)write_data(GREEN);
		else if(i>89)write_data(CYAN);
		else if(i>59)write_data(MAGENTA);
		else if(i>29)write_data(YELLOW);
		else write_data(WHITE);
	}
}

/*************************************************/
int main()
{
	PORTD = 0xF8;	    // pull-up on switches, all others off
	DDRD  = 0x0F;   	// VPP and led pins output

	DDRA = 0xFF;    	// low data port, all outputs
	DDRE = 0xFF;    	// low data port, all outputs
	
	PORTC = 0xFE;	    // all pins high
	DDRC  = 0xFE;	    // all outputs

	uint8 retry = 0;

	OSCCAL = 0x00;    //in order to operate some low speed card the initialization should run at lowest speed

	delay(0xffff);

	MMC_SD_Init();    //SPI initialize

	// Initialize the display
	initialization();

	LCD_test();

	sd_restart: 	//label for "goto"

	while (MMC_SD_Reset())  // Initialize SD Card
	{
		retry++;
		if (retry > 254)
		{
			while (1)
			{
				goto sd_restart;				// try again
			}
		}
	}

	OSCCAL = 0xff;      //run at high speed for normal operation for SD Card

	delay(0xffff);      

	if (FAT_Init())   	//initialize file system - FAT16 and FAT32 are supported
	{
		while (1)
		{
			goto sd_restart;				// try again
		}
	}

	Search(PATH, &PictureInfo, &totalfiles);

	Picture_Slide_Show();

	while (1)
	{
	}
	return 0;

}
/*************************************************/
